// Adapted from https://sourceforge.net/p/dadisi/code/8/tree/trunk/dadisi/
/*
 * File:   Disk.h
 * Author: fermat
 *
 * Created on 20. Januar 2010, 10:39
 */

#ifndef _DISK_H
#define _DISK_H

#define __STDC_LIMIT_MACROS // required for limit macros
#include <stdint.h>
#include <list>
#include <string>

namespace VDRIVE {

/**
 * This class represents a Disk for the Distributor. Each Disk consists of
 * an unique ID (in the list of Disks used by this lib), a capacity and a
 * reference to some data, that can be used by the person instantiating a
 * Disk.
 *
 * @author Sascha Effert <fermat@uni-paderborn.de>
 */
class Disk {
public:
    /**
     * Instantiates a new Disk with the given values.
     *
     * @param id ID of the disk. Has to be unique all over this library.
     * @param capacity Capacity of the Disk in bytes. (This can also be in
     *                 any other scale, but has to be same for ExtentSize
     *                 of Distributor)
     * @param hostname TODO(dauer)
     * @param uri TODO(dauer)
     */
    Disk(int64_t id, int64_t capacity, std::string hostname, std::string uri);

    /**
     * copy constructor
     *
     * @param orig original Disk
     */
    Disk(const Disk& orig);

    /**
     * Instantiates a new Disk with values read from the given ifstream.
     * This is assumed to be written previously by Disk::serialize().
     */
    Disk(std::ifstream& in);

    /**
     * Destructor
     */
    virtual ~Disk();

    /**
     * Get the ID of the disk. Has to be unique all over this library.
     *
     * @return ID of the disk. Has to be unique all over this library.
     */
    int64_t
    getId() const;

    /**
     * Set the ID of the disk. Has to be unique all over this library.
     *
     * @param id ID of the disk. Has to be unique all over this library.
     */
    void
    setId(int64_t id);

    /**
     * Get the Capacity of the Disk in bytes. (This can also be in any
     * other scale, but has to be same for ExtentSize of Distributor)
     *
     * @return Capacity of the Disk in bytes. (This can also be in any
     *         other scale, but has to be same for ExtentSize of
     *         Distributor)
     */
    int64_t
    getCapacity() const;

    /**
     * Set the Capacity of the Disk in bytes. (This can also be in any
     * other scale, but has to be same for ExtentSize of Distributor)
     *
     * @param capacity Capacity of the Disk in bytes. (This can also be in
     *                any other scale, but has to be same for ExtentSize of
     *                Distributor)
     */
    void
    setCapacity(int64_t capacity);

    /**
     */
    std::string
    getHostname() const;

    /**
     * May not be changed by Distributors!
     *
     * @param hostname to be used by developer instantiating this disk.
     *             May not be changed by Distributors!
     */
    void
    setHostname(std::string data);

    std::string
    getUri() const;

    void
    setUri(std::string uri);

    void
    serialize(std::ofstream& out);

private:
    /**
     * ID of the disk. Has to be unique all over this library.
     */
    int64_t id;

    /**
     * Capacity of the Disk in bytes. (This can also be in any other
     * scale, but has to be same for ExtentSize of Distributor)
     */
    int64_t capacity;

    /**
     * String to be used by developer instantiating this disk. May not be
     * changed by Distributors!
     */
    std::string hostname;

    /**
     * String to be used by developer instantiating this disk. May not be
     * changed by Distributors!
     */
    std::string uri;
};
} // namespace VDRIVE
#endif /* _DISK_H */
